/**
 * Theme functions file
 *
 * Contains handlers for navigation, accessibility, header sizing
 * footer widgets and Featured Content slider
 *
 */
( function( $ ) {
	// detect some user agents // add class for specific styles and scripting
	if(navigator.userAgent.match(/Android/i)){
		$("html").addClass("android");
	}
	else if(navigator.userAgent.match(/webOS/i)){
			$("html").addClass("webos");
	}
	else if(navigator.userAgent.match(/iPhone/i)){
			$("html").addClass("iphone");
	}
	else if(navigator.userAgent.match(/iPod/i)){
			$("html").addClass("ipod");
	}
	else if(navigator.userAgent.match(/iPad/i)){
			$("html").addClass("ipad");
	}
	else if(navigator.userAgent.match(/BlackBerry/i)){
			$("html").addClass("blackberry"); 
	}
	
	//Start the scripting
	var body    = $( 'body' ),
		_window = $( window );

	// Enable menu toggle for small screens.
	( function() {
		var nav = $( '#primary-navigation' ), button, menu;
		if ( ! nav ) {
			return;
		}

		button = nav.find( '.menu-toggle' );
		if ( ! button ) {
			return;
		}

		// Hide button if menu is missing or empty.
		menu = nav.find( '.nav-menu' );
		if ( ! menu || ! menu.children().length ) {
			button.hide();
			return;
		}

		$( '.menu-toggle' ).on( 'click.twentyfourteen', function() {
			nav.toggleClass( 'toggled-on' );
		} );
	} )();

	/*
	 * Makes "skip to content" link work correctly in IE9 and Chrome for better
	 * accessibility.
	 *
	 * @link http://www.nczonline.net/blog/2013/01/15/fixing-skip-to-content-links/
	 */
	_window.on( 'hashchange.twentyfourteen', function() {
		var element = document.getElementById( location.hash.substring( 1 ) );

		if ( element ) {
			if ( ! /^(?:a|select|input|button|textarea)$/i.test( element.tagName ) ) {
				element.tabIndex = -1;
			}

			element.focus();

			// Repositions the window on jump-to-anchor to account for header height.
			window.scrollBy( 0, -80 );
		}
	} );

	$( function() {
		// Search toggle.
		$( '.search-toggle' ).on( 'click.twentyfourteen', function( event ) {
			var that    = $( this ),
				wrapper = $( '.search-box-wrapper' );

			that.toggleClass( 'active' );
			wrapper.toggleClass( 'hide' );

			if ( that.is( '.active' ) || $( '.search-toggle .screen-reader-text' )[0] === event.target ) {
				wrapper.find( '.search-field' ).focus();
			}
		} );

		/*
		 * Fixed header for large screen.
		 * If the header becomes more than 48px tall, unfix the header.
		 *
		 * The callback on the scroll event is only added if there is a header
		 * image and we are not on mobile.
		 */
		 
		 
		if ( _window.width() > 781 ) {
			var mastheadHeight = $( '.master-header' ).height(),
				toolbarOffset, mastheadOffset;

			if ( mastheadHeight > 114 ) {
				body.removeClass( 'masthead-fixed' );
			}
    
		
			if ( body.is( '.header-image' ) ) {
				toolbarOffset  = body.is( '.admin-bar' ) ? $( '#wpadminbar' ).height() : 0;
				mastheadOffset = $( '.master-header' ).offset().top - toolbarOffset;
				
				_window.on( 'scroll.twentyfourteen', function() {
					if ( ( window.scrollY > mastheadOffset ) && ( mastheadHeight < 115 ) ) {
						body.addClass( 'masthead-fixed' );
					} else {
						 body.removeClass( 'masthead-fixed' );  
					}
				});
			}
		}
		
		// Focus styles for menus.
		$( '.primary-navigation, .secondary-navigation' ).find( 'a' ).on( 'focus.twentyfourteen blur.twentyfourteen', function() {
			$( this ).parents().toggleClass( 'focus' );
		} );
	} );
	//show / hide top address section
	$('.toggle').on('click', function(e) {
		if (!$(this).hasClass('open')) {
			$(this).addClass('open');
			$(this).children().removeClass('fa-angle-up').addClass('fa-angle-down');
			$('header > section:first-child').slideDown();
		}else{
			$(this).removeClass('open');
			$(this).children().removeClass('fa-angle-down').addClass('fa-angle-up');
			$('header > section:first-child').slideUp();
		}
  });
	_window.load( function() {
		// Arrange footer widgets vertically.
		if ( $.isFunction( $.fn.masonry ) ) {
			$( '#footer-sidebar' ).masonry( {
				itemSelector: '.widget',
				columnWidth: function( containerWidth ) {
					return containerWidth / 4;
				},
				gutterWidth: 0,
				isResizable: true,
				isRTL: $( 'body' ).is( '.rtl' )
			} );
		}

		// Initialize Featured Content slider.
		if ( body.is( '.slider' ) ) {
			$( '.featured-content' ).featuredslider( {
				selector: '.featured-content-inner > article',
				controlsContainer: '.featured-content'
			} );
		}

		// Initialize bxslider.
		if ( body.is( '.home' ) ) {
			$('.homepage-testimonials').bxSlider({
				minSlides: 1,
				maxSlides: 1,
				slideWidth: 670,
				pager: false,
				slideMargin: 30,
				nextText: '<i class="fa fa-angle-right"></i>',
				prevText: '<i class="fa fa-angle-left"></i>'
			});
		}

		//Set some charts, ffs hate this multipling
		if ( body.is( '.home' ) ) {	
			$(function() {
				$('.chart.employees').easyPieChart({
					barColor: '#00bff3',
					trackColor: '#798394',
					scaleColor: false,
					lineWidth: 8,
					easing: 'easeOutBounce',
					onStep: function(from, to, percent) {
						$(this.el).find('.percent').text(Math.round(percent));
					}
				});
				$('.chart.projects').easyPieChart({
					barColor: '#724fa0',
					trackColor: '#798394',
					scaleColor: false,
					lineWidth: 8,
					easing: 'easeOutBounce',
					onStep: function(from, to, percent) {
						$(this.el).find('.percent').text(Math.round(percent));
					}
				});
				$('.chart.sales').easyPieChart({
					barColor: '#f26c4f',
					trackColor: '#798394',
					scaleColor: false,
					lineWidth: 8,
					easing: 'easeOutBounce',
					onStep: function(from, to, percent) {
						$(this.el).find('.percent').text(Math.round(percent));
					}
				});
				$('.chart.stock').easyPieChart({
					barColor: '#30b44a',
					trackColor: '#798394',
					scaleColor: false,
					lineWidth: 8,
					easing: 'easeOutBounce',
					onStep: function(from, to, percent) {
						$(this.el).find('.percent').text(Math.round(percent));
					}
				});
			});
		}
		
		//* Scroll to the top of the skys, but do it slow :) */
		$("a[href='#main']").on('click', function() {
			$("html, body").animate({ scrollTop: 0 }, "slow");
			//console.log(this)
			return false;
		});
		//style 2nd word from tagline
		$(".tagline").children().each(function(){
      var tagline = $(this).text().split(" •");
      tagline[1] = '<span class="highlight">' + tagline[1] + '</span>';
      tagline = tagline.join(" <span>•</span> ");
      $(this).html(tagline);
		});

	} ); //end window load
	
	/*
	 * Change bxslider options based on media querys
	 * @link http://stackoverflow.com/questions/21036861/change-the-of-image-in-bx-slider-on-window-resize
	 */
	 if ( body.is( '.home' ) ) {	
	 //assign objects
   var s;
      var a = { minSlides: 3, maxSlides: 4, slideWidth: 240, pager: false, nextText: '<i class="fa fa-angle-right"></i>', prevText: '<i class="fa fa-angle-left"></i>'},
		      b = { moveSlides: 4 },
				  c = { moveSlides: 1 },
          d = $.extend( {}, a, b, c);
		//do some media check
   $(document).ready(function(){
       if (_window.width() < 992 ) {  
           d = $.extend( {}, a, c);
       } else {
           d = $.extend( {}, a, b);
       }
       s = $('.bxslider').bxSlider(d);
   });
	 //manage the resize
  _window.resize(function() {
       var ww = _window.width();
       if (ww < 992 ) {  
           d = $.extend( {}, a, c);
       } else {
           d = $.extend( {}, a, b);
       }
			 if ( ww ) {
			 		 s.reloadSlider( d );
			 }
   });
	 
	} //endif
} )( jQuery );
